@@ -1,5 +1,5 @@
 # ###################################################
-# Copyright (C) 2011 The Unknown Horizons Team
+# Copyright (C) 2008-2017 The Unknown Horizons Team
 # team@unknown-horizons.org
 # This file is part of Unknown Horizons.
 #
@@ -19,18 +19,20 @@
 # 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # ###################################################
 
-from fakeresourcedeposit import AbstractFakeResourceDeposit
 from horizons.constants import BUILDINGS
 from horizons.util.python import decorators
 
+from .fakeresourcedeposit import AbstractFakeResourceDeposit
+
+
 class AbstractIronDeposit(AbstractFakeResourceDeposit):
 	@classmethod
 	def get_higher_level_building_id(cls):
-		return BUILDINGS.IRON_MINE_CLASS
+		return BUILDINGS.MINE
 
 	@classmethod
 	def register_buildings(cls):
-		cls._available_buildings[BUILDINGS.MOUNTAIN_CLASS] = cls
+		cls._available_buildings[BUILDINGS.MOUNTAIN] = cls
 
 AbstractIronDeposit.register_buildings()
 
