@@ -1,5 +1,5 @@
 # ###################################################
-# Copyright (C) 2011 The Unknown Horizons Team
+# Copyright (C) 2008-2017 The Unknown Horizons Team
 # team@unknown-horizons.org
 # This file is part of Unknown Horizons.
 #
@@ -20,21 +20,29 @@
 # ###################################################
 
 import itertools
-
 from collections import deque
 from functools import partial
 
-from builder import Builder
-from areabuilder import AreaBuilder
-from constants import BUILD_RESULT, BUILDING_PURPOSE
-
+from horizons.ai.aiplayer.basicbuilder import BasicBuilder
 from horizons.command.building import Tear
 from horizons.command.production import ToggleActive
+from horizons.component.namedcomponent import NamedComponent
 from horizons.constants import AI, BUILDINGS
-from horizons.scheduler import Scheduler
-from horizons.util import Callback, Point, Rect
-from horizons.util.python import decorators
 from horizons.entities import Entities
+from horizons.scheduler import Scheduler
+from horizons.util.python import decorators
+from horizons.util.python.callback import Callback
+from horizons.util.shapes import Rect, distances
+from horizons.world.buildability.binarycache import BinaryBuildabilityCache
+from horizons.world.buildability.potentialroadconnectivitycache import \
+	PotentialRoadConnectivityCache
+from horizons.world.buildability.simplecollectorareacache import SimpleCollectorAreaCache
+from horizons.world.building.production import Mine
+from horizons.world.production.producer import Producer
+
+from .areabuilder import AreaBuilder
+from .constants import BUILD_RESULT, BUILDING_PURPOSE
+
 
 class ProductionBuilder(AreaBuilder):
 	"""
@@ -43,7 +51,7 @@ class ProductionBuilder(AreaBuilder):
 	Important attributes:
 	* plan: a dictionary of the form {(x, y): (purpose, extra data), ...} where purpose is one of the BUILDING_PURPOSE constants.
 		Coordinates being in the plan means that the tile doesn't belong to another player.
-	* collector_buildings: a list of every building in the settlement that provides general collectors (branch office, storages)
+	* collector_buildings: a list of every building in the settlement that provides general collectors (warehouse, storages)
 	* production_buildings: a list of buildings in the settlement where productions should be paused and resumed at appropriate times
 	* unused_fields: a dictionary where the key is a BUILDING_PURPOSE constant of a field and the value is a deque that holds the
 		coordinates of unused field spots. {building purpose: deque([(x, y), ...]), ...}
@@ -51,14 +59,17 @@ class ProductionBuilder(AreaBuilder):
 	* last_collector_improvement_road: the last tick when a new road connection was built to improve collector coverage
 	"""
 
-	coastal_building_classes = [BUILDINGS.FISHERMAN_CLASS, BUILDINGS.BOATBUILDER_CLASS, BUILDINGS.SALT_PONDS_CLASS]
+	coastal_building_classes = [BUILDINGS.FISHER, BUILDINGS.BOAT_BUILDER, BUILDINGS.SALT_PONDS]
 
 	def __init__(self, settlement_manager):
 		super(ProductionBuilder, self).__init__(settlement_manager)
 		self.plan = dict.fromkeys(self.land_manager.production, (BUILDING_PURPOSE.NONE, None))
 		self.__init(settlement_manager, Scheduler().cur_tick, Scheduler().cur_tick)
-		for x, y in settlement_manager.settlement.branch_office.position.tuple_iter():
-			self.register_change(x, y, BUILDING_PURPOSE.BRANCH_OFFICE, None)
+		self._init_buildability_cache()
+		self._init_simple_collector_area_cache()
+		self._init_road_connectivity_cache()
+		self.register_change_list(list(settlement_manager.settlement.warehouse.position.tuple_iter()),
+		                          BUILDING_PURPOSE.WAREHOUSE, None)
 		self._refresh_unused_fields()
 
 	def __init(self, settlement_manager, last_collector_improvement_storage, last_collector_improvement_road):
@@ -68,20 +79,41 @@ class ProductionBuilder(AreaBuilder):
 		self.personality = self.owner.personality_manager.get('ProductionBuilder')
 		self.last_collector_improvement_storage = last_collector_improvement_storage
 		self.last_collector_improvement_road = last_collector_improvement_road
-		self.__builder_cache = {}
+
+	def _init_buildability_cache(self):
+		self.buildability_cache = BinaryBuildabilityCache(self.island.terrain_cache)
+		free_coords_set = set()
+		for coords, (purpose, _) in self.plan.iteritems():
+			if purpose == BUILDING_PURPOSE.NONE:
+				free_coords_set.add(coords)
+		for coords in self.land_manager.coastline:
+			free_coords_set.add(coords)
+		self.buildability_cache.add_area(free_coords_set)
+
+	def _init_simple_collector_area_cache(self):
+		self.simple_collector_area_cache = SimpleCollectorAreaCache(self.island.terrain_cache)
+
+	def _init_road_connectivity_cache(self):
+		self.road_connectivity_cache = PotentialRoadConnectivityCache(self)
+		coords_set = set()
+		for coords in self.plan:
+			coords_set.add(coords)
+		for coords in self.land_manager.roads:
+			coords_set.add(coords)
+		self.road_connectivity_cache.modify_area(list(sorted(coords_set)))
 
 	def save(self, db):
 		super(ProductionBuilder, self).save(db)
 		translated_last_collector_improvement_storage = self.last_collector_improvement_storage - Scheduler().cur_tick # pre-translate for the loading process
 		translated_last_collector_improvement_road = self.last_collector_improvement_road - Scheduler().cur_tick # pre-translate for the loading process
-		db("INSERT INTO ai_production_builder(rowid, settlement_manager, last_collector_improvement_storage, last_collector_improvement_road) VALUES(?, ?, ?, ?)", \
+		db("INSERT INTO ai_production_builder(rowid, settlement_manager, last_collector_improvement_storage, last_collector_improvement_road) VALUES(?, ?, ?, ?)",
 			self.worldid, self.settlement_manager.worldid, translated_last_collector_improvement_storage, translated_last_collector_improvement_road)
 		for (x, y), (purpose, _) in self.plan.iteritems():
 			db("INSERT INTO ai_production_builder_plan(production_builder, x, y, purpose) VALUES(?, ?, ?, ?)", self.worldid, x, y, purpose)
 
 	def _load(self, db, settlement_manager):
 		worldid, last_storage, last_road = \
-			db("SELECT rowid, last_collector_improvement_storage, last_collector_improvement_road FROM ai_production_builder WHERE settlement_manager = ?", \
+			db("SELECT rowid, last_collector_improvement_storage, last_collector_improvement_road FROM ai_production_builder WHERE settlement_manager = ?",
 			settlement_manager.worldid)[0]
 		super(ProductionBuilder, self)._load(db, settlement_manager, worldid)
 		self.__init(settlement_manager, last_storage, last_road)
@@ -91,10 +123,13 @@ class ProductionBuilder(AreaBuilder):
 			self.plan[(x, y)] = (purpose, None)
 			if purpose == BUILDING_PURPOSE.ROAD:
 				self.land_manager.roads.add((x, y))
+		self._init_buildability_cache()
+		self._init_simple_collector_area_cache()
+		self._init_road_connectivity_cache()
 		self._refresh_unused_fields()
 
 	def have_deposit(self, resource_id):
-		"""Returns true if there is a resource deposit of the relevant type inside the settlement."""
+		"""Returns True if there is a resource deposit of the relevant type inside the settlement."""
 		for tile in self.land_manager.resource_deposits[resource_id]:
 			if tile.object.settlement is None:
 				continue
@@ -103,24 +138,31 @@ class ProductionBuilder(AreaBuilder):
 				return True
 		return False
 
-	def build_best_option(self, options, purpose):
-		"""
-		Try to build the highest valued option. Return a BUILD_RESULT constant showing how it went.
+	def extend_settlement_with_storage(self, target_position):
+		"""Build a storage to extend the settlement towards the given position. Return a BUILD_RESULT constant."""
+		if not self.have_resources(BUILDINGS.STORAGE):
+			return BUILD_RESULT.NEED_RESOURCES
 
-		@param options: [(value, builder), ...]
-		@param purpose: a BUILDING_PURPOSE constant
-		"""
+		storage_class = Entities.buildings[BUILDINGS.STORAGE]
+		storage_spots = self.island.terrain_cache.get_buildability_intersection(storage_class.terrain_type,
+			storage_class.size, self.settlement.buildability_cache, self.buildability_cache)
+		storage_surrounding_offsets = Rect.get_surrounding_offsets(storage_class.size)
+		coastline = self.land_manager.coastline
 
-		if not options:
-			return BUILD_RESULT.IMPOSSIBLE
+		options = []
+		for (x, y) in sorted(storage_spots):
+			builder = BasicBuilder.create(BUILDINGS.STORAGE, (x, y), 0)
 
-		builder = max(options)[1]
-		if not builder.execute():
-			return BUILD_RESULT.UNKNOWN_ERROR
-		for x, y in builder.position.tuple_iter():
-			self.register_change(x, y, BUILDING_PURPOSE.RESERVED, None)
-		self.register_change(builder.position.origin.x, builder.position.origin.y, purpose, None)
-		return BUILD_RESULT.OK
+			alignment = 1
+			for (dx, dy) in storage_surrounding_offsets:
+				coords = (x + dx, y + dy)
+				if coords in coastline or coords not in self.plan or self.plan[coords][0] != BUILDING_PURPOSE.NONE:
+					alignment += 1
+
+			distance = distances.distance_rect_rect(target_position, builder.position)
+			value = distance - alignment * 0.7
+			options.append((-value, builder))
+		return self.build_best_option(options, BUILDING_PURPOSE.STORAGE)
 
 	def get_collector_area(self):
 		"""Return the set of all coordinates that are reachable from at least one collector by road or open space."""
@@ -141,12 +183,11 @@ class ProductionBuilder(AreaBuilder):
 				queue.append(coords)
 
 			while queue:
-				x, y = queue[0]
-				queue.popleft()
+				x, y = queue.popleft()
 				for dx, dy in moves:
 					coords = (x + dx, y + dy)
 					if coords not in reachable and coords in coverage_area:
-						if coords in self.land_manager.roads or (coords in self.plan and self.plan[coords][0] == BUILDING_PURPOSE.NONE):
+						if coords in self.land_manager.roads or (coords in self.plan and coords not in self.land_manager.coastline and self.plan[coords][0] == BUILDING_PURPOSE.NONE):
 							queue.append(coords)
 							reachable.add(coords)
 							if coords in self.plan and self.plan[coords][0] == BUILDING_PURPOSE.NONE:
@@ -154,7 +195,7 @@ class ProductionBuilder(AreaBuilder):
 		self.__collector_area_cache = (self.last_change_id, collector_area)
 		return collector_area
 
-	def count_available_squares(self, size, max_num = None):
+	def count_available_squares(self, size, max_num=None):
 		"""
 		Count the number of available and usable (covered by collectors) size x size squares.
 
@@ -198,6 +239,7 @@ class ProductionBuilder(AreaBuilder):
 			BUILDING_PURPOSE.PASTURE: deque(),
 			BUILDING_PURPOSE.SUGARCANE_FIELD: deque(),
 			BUILDING_PURPOSE.TOBACCO_FIELD: deque(),
+			BUILDING_PURPOSE.HERBARY: deque(),
 		}
 
 		for coords, (purpose, _) in sorted(self.plan.iteritems()):
@@ -205,9 +247,13 @@ class ProductionBuilder(AreaBuilder):
 			for dx in xrange(3):
 				for dy in xrange(3):
 					coords2 = (coords[0] + dx, coords[1] + dy)
-					object = self.island.ground_map[coords2].object
-					if object is not None and not object.buildable_upon:
+					if coords2 not in self.island.ground_map:
 						usable = False
+					else:
+						object = self.island.ground_map[coords2].object
+						if object is not None and not object.buildable_upon:
+							usable = False
+							break
 			if usable and purpose in self.unused_fields:
 				self.unused_fields[purpose].append(coords)
 
@@ -216,129 +262,40 @@ class ProductionBuilder(AreaBuilder):
 		if not AI.HIGHLIGHT_PLANS:
 			return
 
-		road_colour = (30, 30, 30)
-		fisher_colour = (128, 128, 128)
-		lumberjack_colour = (30, 255, 30)
-		tree_colour = (0, 255, 0)
-		reserved_colour = (0, 0, 128)
-		unknown_colour = (128, 0, 0)
-		farm_colour = (128, 0, 255)
-		potato_field_colour = (255, 0, 128)
-		pasture_colour = (0, 192, 0)
-		weaver_colour = (0, 64, 64)
-		sugarcane_field_colour = (192, 192, 0)
-		distillery_colour = (255, 128, 40)
-		tobacco_field_colour = (64, 64, 0)
-		tobacconist_colour = (128, 64, 40)
-		clay_pit_colour = (0, 64, 0)
-		brickyard_colour = (0, 32, 0)
-		boatbuilder_colour = (163, 73, 164)
-		salt_ponds_colour = (153, 217, 234)
+		tile_colors = {
+			BUILDING_PURPOSE.ROAD:            ( 30,  30,  30),
+			BUILDING_PURPOSE.FISHER:          (128, 128, 128),
+			BUILDING_PURPOSE.LUMBERJACK:      ( 30, 255,  30),
+			BUILDING_PURPOSE.TREE:            (  0, 255,   0),
+			BUILDING_PURPOSE.FARM:            (128,   0, 255),
+			BUILDING_PURPOSE.POTATO_FIELD:    (255,   0, 128),
+			BUILDING_PURPOSE.PASTURE:         (  0, 192,   0),
+			BUILDING_PURPOSE.WEAVER:          (  0,  64,  64),
+			BUILDING_PURPOSE.SUGARCANE_FIELD: (192, 192,   0),
+			BUILDING_PURPOSE.DISTILLERY:      (255, 128,  40),
+			BUILDING_PURPOSE.TOBACCO_FIELD:   ( 64,  64,   0),
+			BUILDING_PURPOSE.TOBACCONIST:     (128,  64,  40),
+			BUILDING_PURPOSE.CLAY_PIT:        (  0,  64,   0),
+			BUILDING_PURPOSE.BRICKYARD:       (  0,  32,   0),
+			BUILDING_PURPOSE.BOAT_BUILDER:    (163,  73, 164),
+			BUILDING_PURPOSE.SALT_PONDS:      (153, 217, 234),
+			BUILDING_PURPOSE.HERBARY:         ( 64, 200,   0),
+			BUILDING_PURPOSE.RESERVED:        (  0,   0, 128),
+		}
+
+		misc_color = (0, 255, 255)
+		unknown_color = (128, 0, 0)
 		renderer = self.session.view.renderer['InstanceRenderer']
 
 		for coords, (purpose, _) in self.plan.iteritems():
 			tile = self.island.ground_map[coords]
-			if purpose == BUILDING_PURPOSE.ROAD:
-				renderer.addColored(tile._instance, *road_colour)
-			elif purpose == BUILDING_PURPOSE.FISHER:
-				renderer.addColored(tile._instance, *fisher_colour)
-			elif purpose == BUILDING_PURPOSE.LUMBERJACK:
-				renderer.addColored(tile._instance, *lumberjack_colour)
-			elif purpose == BUILDING_PURPOSE.TREE:
-				renderer.addColored(tile._instance, *tree_colour)
-			elif purpose == BUILDING_PURPOSE.FARM:
-				renderer.addColored(tile._instance, *farm_colour)
-			elif purpose == BUILDING_PURPOSE.POTATO_FIELD:
-				renderer.addColored(tile._instance, *potato_field_colour)
-			elif purpose == BUILDING_PURPOSE.PASTURE:
-				renderer.addColored(tile._instance, *pasture_colour)
-			elif purpose == BUILDING_PURPOSE.WEAVER:
-				renderer.addColored(tile._instance, *weaver_colour)
-			elif purpose == BUILDING_PURPOSE.SUGARCANE_FIELD:
-				renderer.addColored(tile._instance, *sugarcane_field_colour)
-			elif purpose == BUILDING_PURPOSE.DISTILLERY:
-				renderer.addColored(tile._instance, *distillery_colour)
-			elif purpose == BUILDING_PURPOSE.TOBACCO_FIELD:
-				renderer.addColored(tile._instance, *tobacco_field_colour)
-			elif purpose == BUILDING_PURPOSE.TOBACCONIST:
-				renderer.addColored(tile._instance, *tobacconist_colour)
-			elif purpose == BUILDING_PURPOSE.CLAY_PIT:
-				renderer.addColored(tile._instance, *clay_pit_colour)
-			elif purpose == BUILDING_PURPOSE.BRICKYARD:
-				renderer.addColored(tile._instance, *brickyard_colour)
-			elif purpose == BUILDING_PURPOSE.BOAT_BUILDER:
-				renderer.addColored(tile._instance, *boatbuilder_colour)
-			elif purpose == BUILDING_PURPOSE.SALT_PONDS:
-				renderer.addColored(tile._instance, *salt_ponds_colour)
-			elif purpose == BUILDING_PURPOSE.RESERVED:
-				renderer.addColored(tile._instance, *reserved_colour)
-			else:
-				renderer.addColored(tile._instance, *unknown_colour)
-
-	def __make_new_builder(self, building_id, x, y, needs_collector, orientation):
-		"""Return a Builder object if it is allowed to be built at the location, otherwise return None (not cached)."""
-		coords = (x, y)
-		# quick check to see whether the origin square is allowed to be in the requested place
-		if building_id == BUILDINGS.CLAY_PIT_CLASS or building_id == BUILDINGS.IRON_MINE_CLASS:
-			# clay deposits and mountains are outside the production plan until they are constructed
-			if coords in self.plan or coords not in self.settlement.ground_map:
-				return None
-		elif building_id in self.coastal_building_classes:
-			# coastal buildings can use coastal tiles
-			if coords not in self.land_manager.coastline and coords in self.plan and self.plan[coords][0] != BUILDING_PURPOSE.NONE:
-				return None
-		else:
-			if coords not in self.plan or self.plan[coords][0] != BUILDING_PURPOSE.NONE or coords not in self.settlement.ground_map:
-				return None
-
-		# create the builder, make sure that it is allowed according to the game logic
-		builder = Builder.create(building_id, self.land_manager, Point(x, y), orientation=orientation)
-		if not builder or not self.land_manager.legal_for_production(builder.position):
-			return None
-
-		# make sure that the position of the building is allowed according to the plan
-		if building_id in self.coastal_building_classes:
-			# coastal buildings can use coastal tiles
-			for coords in builder.position.tuple_iter():
-				if coords not in self.land_manager.coastline and coords in self.plan and self.plan[coords][0] != BUILDING_PURPOSE.NONE:
-					return None
-		elif building_id in [BUILDINGS.CLAY_PIT_CLASS, BUILDINGS.IRON_MINE_CLASS]:
-			# clay deposits and mountains can't be in areas restricted by the plan
-			pass
-		else:
-			for coords in builder.position.tuple_iter():
-				if coords not in self.plan or self.plan[coords][0] != BUILDING_PURPOSE.NONE:
-					return None
-
-		# make sure the building is close enough to a collector if it produces any resources that have to be collected
-		if needs_collector and not any(True for building in self.collector_buildings if building.position.distance(builder.position) <= building.radius):
-			return None
-		return builder
-
-	def make_builder(self, building_id, x, y, needs_collector, orientation = 0):
-		"""Return a Builder object if it is allowed to be built at the location, otherwise return None (cached)."""
-		coords = (x, y)
-		key = (building_id, coords, needs_collector, orientation)
-		size = Entities.buildings[building_id].size
-		if orientation == 1 or orientation == 3:
-			size = (size[1], size[0])
-		if coords not in self.island.last_changed[size]:
-			return None
-
-		island_changed = self.island.last_changed[size][coords]
-		if key in self.__builder_cache and island_changed != self.__builder_cache[key][0]:
-			del self.__builder_cache[key]
-
-		plan_changed = self.last_change_id
-		if key in self.__builder_cache and plan_changed != self.__builder_cache[key][1]:
-			del self.__builder_cache[key]
-
-		if key not in self.__builder_cache:
-			self.__builder_cache[key] = (island_changed, plan_changed, self.__make_new_builder(building_id, x, y, needs_collector, orientation))
-		return self.__builder_cache[key][2]
+			color = tile_colors.get(purpose, misc_color)
+			if purpose == BUILDING_PURPOSE.NONE:
+				color = unknown_color
+			renderer.addColored(tile._instance, *color)
 
 	def _init_cache(self):
-		"""Initialise the cache that knows the last time the buildability of a rectangle may have changed in this area.""" 
+		"""Initialize the cache that knows the last time the buildability of a rectangle may have changed in this area."""
 		super(ProductionBuilder, self)._init_cache()
 		self.__collector_area_cache = None
 		self.__available_squares_cache = {}
@@ -351,13 +308,38 @@ class ProductionBuilder(AreaBuilder):
 			return
 		self.last_change_id += 1
 
+	def register_change_list(self, coords_list, purpose, data):
+		add_list = []
+		remove_list = []
+		for coords in coords_list:
+			if coords in self.land_manager.village or coords not in self.plan:
+				continue
+			if purpose == BUILDING_PURPOSE.NONE and self.plan[coords][0] != BUILDING_PURPOSE.NONE:
+				add_list.append(coords)
+			elif purpose != BUILDING_PURPOSE.NONE and self.plan[coords][0] == BUILDING_PURPOSE.NONE:
+				remove_list.append(coords)
+		if add_list:
+			self.buildability_cache.add_area(add_list)
+		if remove_list:
+			self.buildability_cache.remove_area(remove_list)
+
+		super(ProductionBuilder, self).register_change_list(coords_list, purpose, data)
+		self.road_connectivity_cache.modify_area(coords_list)
+		self.display()
+
 	def handle_lost_area(self, coords_list):
 		"""Handle losing the potential land in the given coordinates list."""
 		# remove planned fields that are now impossible
-		field_size = Entities.buildings[BUILDINGS.POTATO_FIELD_CLASS].size
+		lost_coords_list = []
+		for coords in coords_list:
+			if coords in self.plan:
+				lost_coords_list.append(coords)
+		self.register_change_list(lost_coords_list, BUILDING_PURPOSE.NONE, None)
+
+		field_size = Entities.buildings[BUILDINGS.POTATO_FIELD].size
 		removed_list = []
 		for coords, (purpose, _) in self.plan.iteritems():
-			if purpose in [BUILDING_PURPOSE.POTATO_FIELD, BUILDING_PURPOSE.PASTURE, BUILDING_PURPOSE.SUGARCANE_FIELD, BUILDING_PURPOSE.TOBACCO_FIELD]:
+			if purpose in [BUILDING_PURPOSE.POTATO_FIELD, BUILDING_PURPOSE.PASTURE, BUILDING_PURPOSE.SUGARCANE_FIELD, BUILDING_PURPOSE.TOBACCO_FIELD, BUILDING_PURPOSE.HERBARY]:
 				rect = Rect.init_from_topleft_and_size_tuples(coords, field_size)
 				for field_coords in rect.tuple_iter():
 					if field_coords not in self.land_manager.production:
@@ -366,27 +348,30 @@ class ProductionBuilder(AreaBuilder):
 
 		for coords in removed_list:
 			rect = Rect.init_from_topleft_and_size_tuples(coords, field_size)
-			for field_coords in rect.tuple_iter():
-				self.plan[field_coords] = (BUILDING_PURPOSE.NONE, None)
+			self.register_change_list(list(rect.tuple_iter()), BUILDING_PURPOSE.NONE, None)
 		self._refresh_unused_fields()
 		super(ProductionBuilder, self).handle_lost_area(coords_list)
+		self.road_connectivity_cache.modify_area(lost_coords_list)
 
 	def handle_new_area(self):
 		"""Handle receiving more land to the production area (this can happen when the village area gives some up)."""
+		new_coords_list = []
 		for coords in self.land_manager.production:
 			if coords not in self.plan:
-				self.plan[coords] = (BUILDING_PURPOSE.NONE, None)
+				new_coords_list.append(coords)
+		self.register_change_list(new_coords_list, BUILDING_PURPOSE.NONE, None)
 
-	collector_building_classes = [BUILDINGS.BRANCH_OFFICE_CLASS, BUILDINGS.STORAGE_CLASS]
-	field_building_classes = [BUILDINGS.POTATO_FIELD_CLASS, BUILDINGS.PASTURE_CLASS, BUILDINGS.SUGARCANE_FIELD_CLASS, BUILDINGS.TOBACCO_FIELD_CLASS]
-	production_building_classes = set([BUILDINGS.FISHERMAN_CLASS, BUILDINGS.LUMBERJACK_CLASS, BUILDINGS.FARM_CLASS, BUILDINGS.CLAY_PIT_CLASS,
-		BUILDINGS.BRICKYARD_CLASS, BUILDINGS.WEAVER_CLASS, BUILDINGS.DISTILLERY_CLASS, BUILDINGS.IRON_MINE_CLASS, BUILDINGS.SMELTERY_CLASS,
-		BUILDINGS.TOOLMAKER_CLASS, BUILDINGS.CHARCOAL_BURNER_CLASS, BUILDINGS.TOBACCONIST_CLASS, BUILDINGS.SALT_PONDS_CLASS])
+	collector_building_classes = [BUILDINGS.WAREHOUSE, BUILDINGS.STORAGE]
+	field_building_classes = [BUILDINGS.POTATO_FIELD, BUILDINGS.PASTURE, BUILDINGS.SUGARCANE_FIELD, BUILDINGS.TOBACCO_FIELD]
+	production_building_classes = set([BUILDINGS.FISHER, BUILDINGS.LUMBERJACK, BUILDINGS.FARM, BUILDINGS.CLAY_PIT,
+		BUILDINGS.BRICKYARD, BUILDINGS.WEAVER, BUILDINGS.DISTILLERY, BUILDINGS.MINE, BUILDINGS.SMELTERY,
+		BUILDINGS.TOOLMAKER, BUILDINGS.CHARCOAL_BURNER, BUILDINGS.TOBACCONIST, BUILDINGS.SALT_PONDS])
 
 	def add_building(self, building):
 		"""Called when a new building is added in the area (the building already exists during the call)."""
 		if building.id in self.collector_building_classes:
 			self.collector_buildings.append(building)
+			self.simple_collector_area_cache.add_building(building)
 		elif building.id in self.production_building_classes:
 			self.production_buildings.append(building)
 
@@ -394,23 +379,26 @@ class ProductionBuilder(AreaBuilder):
 
 	def _handle_lumberjack_removal(self, building):
 		"""Release the unused trees around the lumberjack building being removed."""
-		used_trees = set()
-		for lumberjack_building in self.settlement.get_buildings_by_id(BUILDINGS.LUMBERJACK_CLASS):
+		trees_used_by_others = set()
+		for lumberjack_building in self.settlement.buildings_by_id.get(BUILDINGS.LUMBERJACK, []):
 			if lumberjack_building.worldid == building.worldid:
 				continue
 			for coords in lumberjack_building.position.get_radius_coordinates(lumberjack_building.radius):
-				used_trees.add(coords)
+				if coords in self.plan and self.plan[coords][0] == BUILDING_PURPOSE.TREE:
+					trees_used_by_others.add(coords)
 
+		coords_list = []
 		for coords in building.position.get_radius_coordinates(building.radius):
-			if coords not in used_trees and coords in self.plan and self.plan[coords][0] == BUILDING_PURPOSE.TREE:
-				self.register_change(coords[0], coords[1], BUILDING_PURPOSE.NONE, None)
+			if coords not in trees_used_by_others and coords in self.plan and self.plan[coords][0] == BUILDING_PURPOSE.TREE:
+				coords_list.append(coords)
+		self.register_change_list(coords_list, BUILDING_PURPOSE.NONE, None)
 
 	def _handle_farm_removal(self, building):
 		"""Handle farm removal by removing planned fields and tearing existing ones that can't be serviced by another farm."""
 		unused_fields = set()
-		farms = self.settlement.get_buildings_by_id(BUILDINGS.FARM_CLASS)
+		farms = self.settlement.buildings_by_id.get(BUILDINGS.FARM, [])
 		for coords in building.position.get_radius_coordinates(building.radius):
-			if not coords in self.plan:
+			if coords not in self.plan:
 				continue
 			object = self.island.ground_map[coords].object
 			if object is None or object.id not in self.field_building_classes:
@@ -426,15 +414,14 @@ class ProductionBuilder(AreaBuilder):
 
 		# tear the finished but no longer used fields down
 		for unused_field in unused_fields:
-			for x, y in unused_field.position.tuple_iter():
-				self.register_change(x, y, BUILDING_PURPOSE.NONE, None)
+			self.register_change_list(list(unused_field.position.tuple_iter()), BUILDING_PURPOSE.NONE, None)
 			Tear(unused_field).execute(self.session)
 
 		# remove the planned but never built fields from the plan
 		self._refresh_unused_fields()
 		for unused_fields_list in self.unused_fields.itervalues():
 			for coords in unused_fields_list:
-				position = Rect.init_from_topleft_and_size_tuples(coords, Entities.buildings[BUILDINGS.POTATO_FIELD_CLASS].size)
+				position = Rect.init_from_topleft_and_size_tuples(coords, Entities.buildings[BUILDINGS.POTATO_FIELD].size)
 				if building.position.distance(position) > building.radius:
 					continue # it never belonged to the removed building
 
@@ -444,54 +431,62 @@ class ProductionBuilder(AreaBuilder):
 						used_by_another_farm = True
 						break
 				if not used_by_another_farm:
-					for x, y in position.tuple_iter():
-						self.register_change(x, y, BUILDING_PURPOSE.NONE, None)
+					self.register_change_list(list(position.tuple_iter()), BUILDING_PURPOSE.NONE, None)
 		self._refresh_unused_fields()
 
 	def remove_building(self, building):
 		"""Called when a building is removed from the area (the building still exists during the call)."""
 		if building.id in self.field_building_classes:
 			# this can't be handled right now because the building still exists
-			Scheduler().add_new_object(Callback(self._refresh_unused_fields), self, run_in = 0)
-			Scheduler().add_new_object(Callback(partial(super(ProductionBuilder, self).remove_building, building)), self, run_in = 0)
-		elif building.buildable_upon or building.id == BUILDINGS.TRAIL_CLASS:
-			pass # don't react to road, trees and tent ruins being destroyed
+			Scheduler().add_new_object(Callback(self._refresh_unused_fields), self, run_in=0)
+			Scheduler().add_new_object(Callback(partial(super(ProductionBuilder, self).remove_building, building)), self, run_in=0)
+		elif building.buildable_upon or building.id == BUILDINGS.TRAIL:
+			pass # don't react to road, trees and ruined tents being destroyed
 		else:
-			for x, y in building.position.tuple_iter():
-				self.register_change(x, y, BUILDING_PURPOSE.NONE, None)
+			self.register_change_list(list(building.position.tuple_iter()), BUILDING_PURPOSE.NONE, None)
 
 			if building.id in self.collector_building_classes:
 				self.collector_buildings.remove(building)
+				self.simple_collector_area_cache.remove_building(building)
 			elif building.id in self.production_building_classes:
 				self.production_buildings.remove(building)
 
-			if building.id == BUILDINGS.LUMBERJACK_CLASS:
+			if building.id == BUILDINGS.LUMBERJACK:
 				self._handle_lumberjack_removal(building)
-			elif building.id == BUILDINGS.FARM_CLASS:
+			elif building.id == BUILDINGS.FARM:
 				self._handle_farm_removal(building)
 
 			super(ProductionBuilder, self).remove_building(building)
 
 	def manage_production(self):
-		"""Pauses and resumes production buildings when they have full output inventories."""
+		"""Pauses and resumes production buildings when they have full input and output inventories."""
 		for building in self.production_buildings:
-			for production in building._get_productions():
+			producer = building.get_component(Producer)
+			for production in producer.get_productions():
+				if not production.get_produced_resources():
+					continue
 				all_full = True
 
 				# inventory full of the produced resources?
-				to_check = production._prod_line.production if building.id != BUILDINGS.CLAY_PIT_CLASS else production.get_produced_res()
-				for resource_id in to_check:
-					if production.inventory.get_free_space_for(resource_id) > 0:
+				for resource_id, min_amount in production.get_produced_resources().iteritems():
+					if production.inventory.get_free_space_for(resource_id) >= min_amount:
 						all_full = False
 						break
 
+				# inventory full of the input resource?
+				if all_full and not isinstance(building, Mine):
+					for resource_id in production.get_consumed_resources():
+						if production.inventory.get_free_space_for(resource_id) > 0:
+							all_full = False
+							break
+
 				if all_full:
 					if not production.is_paused():
-						ToggleActive(building, production).execute(self.land_manager.session)
+						ToggleActive(producer, production).execute(self.land_manager.session)
 						self.log.info('%s paused a production at %s/%d', self, building.name, building.worldid)
 				else:
 					if production.is_paused():
-						ToggleActive(building, production).execute(self.land_manager.session)
+						ToggleActive(producer, production).execute(self.land_manager.session)
 						self.log.info('%s resumed a production at %s/%d', self, building.name, building.worldid)
 
 	def handle_mine_empty(self, mine):
@@ -499,6 +494,7 @@ class ProductionBuilder(AreaBuilder):
 		self.land_manager.refresh_resource_deposits()
 
 	def __str__(self):
-		return '%s.PB(%s/%d)' % (self.owner, self.settlement.name if hasattr(self, 'settlement') else 'unknown', self.worldid)
+		return '%s.PB(%s/%s)' % (self.owner, self.settlement.get_component(NamedComponent).name if hasattr(self, 'settlement') else 'unknown',
+			self.worldid if hasattr(self, 'worldid') else 'none')
 
-decorators.bind_all(ProductionBuilder)
\ No newline at end of file
+decorators.bind_all(ProductionBuilder, stoplist=['AI'])
