@@ -1,5 +1,5 @@
 # ###################################################
-# Copyright (C) 2011 The Unknown Horizons Team
+# Copyright (C) 2008-2017 The Unknown Horizons Team
 # team@unknown-horizons.org
 # This file is part of Unknown Horizons.
 #
@@ -21,7 +21,12 @@
 
 """Modules for gui/ingamegui"""
 
-from playerdataselection import PlayerDataSelection
-from aidataselection import AIDataSelection
-from singleplayermenu import SingleplayerMenu
-from multiplayermenu import MultiplayerMenu
+from .playerdataselection import PlayerDataSelection
+from .aidataselection import AIDataSelection
+from .singleplayermenu import SingleplayerMenu
+from .multiplayermenu import MultiplayerMenu
+from .help import HelpDialog
+from .select_savegame import SelectSavegameDialog
+from .loadingscreen import LoadingScreen
+from .pausemenu import PauseMenu
+from .settings import SettingsDialog
