@@ -1,5 +1,5 @@
 # ###################################################
-# Copyright (C) 2011 The Unknown Horizons Team
+# Copyright (C) 2008-2017 The Unknown Horizons Team
 # team@unknown-horizons.org
 # This file is part of Unknown Horizons.
 #
@@ -19,9 +19,13 @@
 # 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # ###################################################
 
-from buildingtool import BuildingTool
-from cursortool import CursorTool
-from navigationtool import NavigationTool
-from selectiontool import SelectionTool
-from tearingtool import TearingTool
-from attackingtool import AttackingTool
+from .buildingtool import BuildingTool
+from .cursortool import CursorTool
+from .navigationtool import NavigationTool
+from .selectiontool import SelectionTool
+from .tearingtool import TearingTool
+from .attackingtool import AttackingTool
+from .pipettetool import PipetteTool
+
+# Editor tools
+from .tilelayingtool import TileLayingTool
