@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # ###################################################
-# Copyright (C) 2011 The Unknown Horizons Team
+# Copyright (C) 2008-2017 The Unknown Horizons Team
 # team@unknown-horizons.org
 # This file is part of Unknown Horizons.
 #
@@ -21,20 +21,16 @@
 # ###################################################
 
 import logging
-import copy
-
 from collections import defaultdict, deque
 
-from horizons.util import WorldObject
-from horizons.util.changelistener import metaChangeListenerDecorator
 from horizons.constants import PRODUCTION
-from horizons.world.production.productionline import ProductionLine
-
 from horizons.scheduler import Scheduler
+from horizons.util.changelistener import ChangeListener, metaChangeListenerDecorator
+from horizons.world.production.productionline import ProductionLine
 
 
 @metaChangeListenerDecorator("production_finished")
-class Production(WorldObject):
+class Production(ChangeListener):
 	"""Class for production to be used by ResourceHandler.
 	Controls production and starts it by watching the assigned building's inventory,
 	which is virtually the only "interface" to the building.
@@ -49,47 +45,58 @@ class Production(WorldObject):
 	can be observed via ChangeListener interface."""
 	log = logging.getLogger('world.production')
 
-	# optimisation:
+	# optimization:
 	# the special resource gold is only stored in the player's inventory.
 	# If productions want to use it, they will observer every change of it, which results in
 	# a lot calls. Therefore, this is not done by default but only for few subclasses that actually need it.
-	USES_GOLD = False
+	uses_gold = False
+
+	keep_original_prod_line = False
 
 	## INIT/DESTRUCT
-	def __init__(self, inventory, owner_inventory, prod_line_id, auto_start=True, **kwargs):
+	def __init__(self, inventory, owner_inventory, prod_id, prod_data,
+	             start_finished=False, load=False, **kwargs):
+		"""
+		@param inventory: interface to the world, take res from here and put output back there
+		@param owner_inventory: same as inventory, but for gold. Usually the players'.
+		@param prod_id: int id of the production line
+		@param prod_data: ?
+		@param start_finished: Whether to start at the final state of a production
+		@param load: set to True if this production is supposed to load a saved production
+		"""
 		super(Production, self).__init__(**kwargs)
+		# this has grown to be a bit weird compared to other init/loads
+		# __init__ is always called before load, therefore load just overwrites some of the values here
 		self._state_history = deque()
-		self.__init(inventory, owner_inventory, prod_line_id, PRODUCTION.STATES.none, Scheduler().cur_tick)
-
-		# don't set call_listener_now to true, adding/removing changelisteners wouldn't be atomic any more
-		self.inventory.add_change_listener(self._check_inventory, call_listener_now=False)
-		if self.__class__.USES_GOLD:
-			self.owner_inventory.add_change_listener(self._check_inventory, call_listener_now=False)
-
-		if auto_start:
-			self._check_inventory()
-
-	def __init(self, inventory, owner_inventory, prod_line_id, state, creation_tick, pause_old_state = None):
-		"""
-		@param inventory: inventory of assigned building
-		@param prod_line_id: id of production line.
-		"""
+		self.prod_id = prod_id
+		self.prod_data = prod_data
+		self.__start_finished = start_finished
 		self.inventory = inventory
 		self.owner_inventory = owner_inventory
-		self._state = state
+
 		self._pause_remaining_ticks = None # only used in pause()
-		self._pause_old_state = pause_old_state # only used in pause()
-		self._creation_tick = creation_tick
+		self._pause_old_state = None # only used in pause()
 
-		assert isinstance(prod_line_id, int)
-		self._prod_line = self._create_production_line(prod_line_id)
+		self._creation_tick = Scheduler().cur_tick
 
-	@classmethod
-	def _create_production_line(self, prod_line_id):
-		"""Returns a non-changeable production line instance"""
-		return ProductionLine.get_const_production_line(prod_line_id)
+		assert isinstance(prod_id, int)
+		self._prod_line = ProductionLine(id=prod_id, data=prod_data)
 
-	def save(self, db):
+		if self.__class__.keep_original_prod_line: # used by unit productions
+			self.original_prod_line = self._prod_line.get_original_copy()
+
+		if not load:
+			# init production to start right away
+
+			if self.__start_finished:
+				# finish the production
+				self._give_produced_res()
+
+			self._state = PRODUCTION.STATES.waiting_for_res
+			self._add_listeners(check_now=True)
+
+	def save(self, db, owner_id):
+		"""owner_id: worldid of the owner of the producer object that owns this production"""
 		self._clean_state_history()
 		current_tick = Scheduler().cur_tick
 		translated_creation_tick = self._creation_tick - current_tick + 1 #  pre-translate the tick number for the loading process
@@ -102,48 +109,42 @@ class Production(WorldObject):
 		# use a number > 0 for ticks
 		if remaining_ticks < 1:
 			remaining_ticks = 1
-		db('INSERT INTO production(rowid, state, prod_line_id, remaining_ticks, _pause_old_state, creation_tick) VALUES(?, ?, ?, ?, ?, ?)', \
-			 self.worldid, self._state.index, self._prod_line.id, remaining_ticks, \
-			 None if self._pause_old_state is None else self._pause_old_state.index, translated_creation_tick)
+		db('INSERT INTO production(rowid, state, prod_line_id, remaining_ticks, \
+		      _pause_old_state, creation_tick, owner) VALUES(?, ?, ?, ?, ?, ?, ?)',
+		         None, self._state.index, self._prod_line.id, remaining_ticks,
+		         None if self._pause_old_state is None else self._pause_old_state.index,
+			    translated_creation_tick, owner_id)
 
 		# save state history
 		for tick, state in self._state_history:
 				# pre-translate the tick number for the loading process
 			translated_tick = tick - current_tick + 1
-			db("INSERT INTO production_state_history(production, tick, state) VALUES(?, ?, ?)", \
-				 self.worldid, translated_tick, state)
+			db("INSERT INTO production_state_history(production, tick, state, object_id) VALUES(?, ?, ?, ?)",
+				 self.prod_id, translated_tick, state, owner_id)
 
-	@classmethod
-	def load(cls, db, worldid):
-		self = cls.__new__(cls)
-		self._load(db, worldid)
-		return self
-
-	def _load(self, db, worldid):
+	def load(self, db, worldid):
+		# NOTE: __init__ must have been called with load=True
+		# worldid is the world id of the producer component instance calling this
 		super(Production, self).load(db, worldid)
 
-		db_data = db.get_production_row(worldid)
-		obj = WorldObject.get_object_by_id(db_data[1])
-		owner_inventory = obj._get_owner_inventory()
-		self.__init(obj.inventory, owner_inventory, db_data[2], PRODUCTION.STATES[db_data[0]], \
-			db_data[5], None if db_data[4] is None else PRODUCTION.STATES[db_data[4]])
+		db_data = db.get_production_by_id_and_owner(self.prod_id, worldid)
+		self._creation_tick = db_data[5]
+		self._state = PRODUCTION.STATES[db_data[0]]
+		self._pause_old_state = None if db_data[4] is None else PRODUCTION.STATES[db_data[4]]
 		if self._state == PRODUCTION.STATES.paused:
 			self._pause_remaining_ticks = db_data[3]
 		elif self._state == PRODUCTION.STATES.producing:
 			Scheduler().add_new_object(self._get_producing_callback(), self, db_data[3])
 		elif self._state == PRODUCTION.STATES.waiting_for_res or \
-				 self._state == PRODUCTION.STATES.inventory_full:
-			self.inventory.add_change_listener(self._check_inventory)
-			if self.__class__.USES_GOLD:
-				self.owner_inventory.add_change_listener(self._check_inventory)
+		     self._state == PRODUCTION.STATES.inventory_full:
+			# no need to call now, this just restores the state before
+			# saving, where it hasn't triggered yet, therefore it won't now
+			self._add_listeners()
 
-		self._state_history = db.get_production_state_history(worldid)
+		self._state_history = db.get_production_state_history(worldid, self.prod_id)
 
 	def remove(self):
-		# depending on state, a check_inventory listener might be active
-		self.inventory.discard_change_listener(self._check_inventory)
-		if self.__class__.USES_GOLD:
-			self.owner_inventory.discard_change_listener(self._check_inventory)
+		self._remove_listeners()
 		Scheduler().rem_all_classinst_calls(self)
 		super(Production, self).remove()
 
@@ -156,7 +157,7 @@ class Production(WorldObject):
 		"""Res that are consumed here. Returns dict {res:amount}. Interface for _prod_line."""
 		return self._prod_line.consumed_res
 
-	def get_produced_res(self):
+	def get_produced_resources(self):
 		"""Res that are produced here. Returns dict {res:amount}. Interface for _prod_line."""
 		return self._prod_line.produced_res
 
@@ -192,7 +193,7 @@ class Production(WorldObject):
 	def is_paused(self):
 		return self._state == PRODUCTION.STATES.paused
 
-	def pause(self, pause = True):
+	def pause(self, pause=True):
 		self.log.debug("Production pause: %s", pause)
 		if not pause: # do unpause
 			# switch state
@@ -200,19 +201,16 @@ class Production(WorldObject):
 			self._pause_old_state = None
 
 			# apply state
-			if self._state in (PRODUCTION.STATES.waiting_for_res, \
-												 PRODUCTION.STATES.inventory_full):
+			if self._state in (PRODUCTION.STATES.waiting_for_res,
+			                   PRODUCTION.STATES.inventory_full,
+			                   PRODUCTION.STATES.done):
 				# just restore watching
-				# don't set call_listener_now to true, adding/removing changelisteners wouldn't be atomic any more
-				self.inventory.add_change_listener(self._check_inventory)
-				if self.__class__.USES_GOLD:
-					self.owner_inventory.add_change_listener(self._check_inventory)
-				self._check_inventory()
+				self._add_listeners(check_now=True)
 
 			elif self._state == PRODUCTION.STATES.producing:
 				# restore scheduler call
-				Scheduler().add_new_object(self._get_producing_callback(), self, \
-																   self._pause_remaining_ticks)
+				Scheduler().add_new_object(self._get_producing_callback(), self,
+				                           self._pause_remaining_ticks)
 			else:
 				assert False, 'Unhandled production state: %s' % self._pause_old_state
 		else: # do pause
@@ -220,19 +218,17 @@ class Production(WorldObject):
 			self._pause_old_state = self._state
 			self._state = PRODUCTION.STATES.paused
 
-			if self._pause_old_state in (PRODUCTION.STATES.waiting_for_res, \
-												           PRODUCTION.STATES.inventory_full):
-				# just stop watching for new res
-				self.inventory.discard_change_listener(self._check_inventory)
-				if self.__class__.USES_GOLD:
-					self.owner_inventory.discard_change_listener(self._check_inventory)
+			if self._pause_old_state in (PRODUCTION.STATES.waiting_for_res,
+			                             PRODUCTION.STATES.inventory_full,
+			                             PRODUCTION.STATES.done):
+				self._remove_listeners()
 			elif self._pause_old_state == PRODUCTION.STATES.producing:
 				# save when production finishes and remove that call
 				self._pause_remaining_ticks = \
 						Scheduler().get_remaining_ticks(self, self._get_producing_callback())
 				Scheduler().rem_call(self, self._get_producing_callback())
 			else:
-				assert False, 'Unhandled production state: %s' % self._state
+				assert False, 'Unhandled production state: %s' % self._pause_old_state
 
 		self._changed()
 
@@ -250,16 +246,13 @@ class Production(WorldObject):
 		except AttributeError: # production line doesn't have this alter method
 			pass
 
-	def get_state_history_length(self):
-		return min(PRODUCTION.STATISTICAL_WINDOW, Scheduler().cur_tick - self._creation_tick)
 
 	def get_state_history_times(self, ignore_pause):
 		"""
 		Returns the part of time 0 <= x <= 1 the production has been in a state during the last history_length ticks.
 		"""
-
 		self._clean_state_history()
-		result = defaultdict(lambda: 0)
+		result = defaultdict(int)
 		current_tick = Scheduler().cur_tick
 		pause_state = PRODUCTION.STATES.paused.index
 		first_relevant_tick = self._get_first_relevant_tick(ignore_pause)
@@ -291,6 +284,14 @@ class Production(WorldObject):
 	def get_age(self):
 		return Scheduler().cur_tick - self._creation_tick
 
+	def get_unstorable_produced_res(self):
+		"""Returns all produced res for whose there is no space"""
+		l = []
+		for res, amount in self._prod_line.produced_res.iteritems():
+			if self.inventory.get_free_space_for(res) < amount:
+				l.append(res)
+		return l
+
 	## PROTECTED METHODS
 	def _get_producing_callback(self):
 		"""Returns the callback used during the process of producing (state: producing)"""
@@ -298,12 +299,14 @@ class Production(WorldObject):
 
 	def _get_first_relevant_tick(self, ignore_pause):
 		"""
-		Returns the first tick that is relevant for production utilisation calculation
+		Returns the first tick that is relevant for production utilization calculation
 		@param ignore_pause: whether to ignore the time spent in the pause state
 		"""
 
 		current_tick = Scheduler().cur_tick
-		first_relevant_tick = current_tick - self.get_state_history_length()
+		state_hist_len = min(PRODUCTION.STATISTICAL_WINDOW, current_tick - self._creation_tick)
+
+		first_relevant_tick = current_tick - state_hist_len
 		if not ignore_pause:
 			return first_relevant_tick
 
@@ -350,9 +353,7 @@ class Production(WorldObject):
 		elif self._check_available_res():
 			# we have space in our inventory and needed res are available
 			# stop listening for res
-			self.inventory.remove_change_listener(self._check_inventory)
-			if self.__class__.USES_GOLD:
-				self.owner_inventory.remove_change_listener(self._check_inventory)
+			self._remove_listeners()
 			self._start_production()
 		else:
 			# we have space in our inventory, but needed res are missing
@@ -367,13 +368,14 @@ class Production(WorldObject):
 
 	def _produce(self):
 		"""Called when there are enough res in the inventory for starting production"""
-		self.log.debug("%s start", self)
+		self.log.debug("%s _produce", self)
 		assert self._check_available_res() and self._check_for_space_for_produced_res()
 		# take the res we need
 		self._remove_res_to_expend()
 		# call finished in some time
 		time = Scheduler().get_ticks(self._prod_line.time)
-		Scheduler().add_new_object(self._finished_producing, self, time)
+		Scheduler().add_new_object(self._get_producing_callback(), self, time)
+		self.log.debug("%s _produce Adding callback in %d time", self, time)
 
 	def _finished_producing(self, continue_producing=True, **kwargs):
 		"""Called when the production finishes."""
@@ -381,22 +383,33 @@ class Production(WorldObject):
 		self._give_produced_res()
 		self.on_production_finished()
 		if continue_producing:
-			self.state = PRODUCTION.STATES.waiting_for_res
-			# don't set call_listener_now to true, adding/removing changelisteners wouldn't be atomic any more
-			self.inventory.add_change_listener(self._check_inventory)
-			if self.__class__.USES_GOLD:
-				self.owner_inventory.add_change_listener(self._check_inventory)
+			self._state = PRODUCTION.STATES.waiting_for_res
+			self._add_listeners(check_now=True)
+
+	def _add_listeners(self, check_now=False):
+		"""Listen for changes in the inventory from now on."""
+		# don't set call_listener_now to True here, adding/removing changelisteners wouldn't be atomic any more
+		self.inventory.add_change_listener(self._check_inventory)
+		if self.__class__.uses_gold:
+			self.owner_inventory.add_change_listener(self._check_inventory)
+
+		if check_now: # only check now after adding everything
 			self._check_inventory()
 
+	def _remove_listeners(self):
+		# depending on state, a check_inventory listener might be active
+		self.inventory.discard_change_listener(self._check_inventory)
+		if self.__class__.uses_gold:
+			self.owner_inventory.discard_change_listener(self._check_inventory)
+
 	def _give_produced_res(self):
 		"""Put produces goods to the inventory"""
 		for res, amount in self._prod_line.produced_res.iteritems():
 			self.inventory.alter(res, amount)
-			self.log.debug("produced %s of %s", amount, res)
 
 	def _check_available_res(self):
 		"""Checks if all required resources are there.
-		@return: bool, true if we can start production
+		@return: bool, True if we can start production
 		"""
 		for res, amount in self._prod_line.consumed_res.iteritems():
 			if self.inventory[res] < (-amount): # consumed res have negative sign
@@ -411,7 +424,7 @@ class Production(WorldObject):
 
 	def _check_for_space_for_produced_res(self):
 		"""Checks if there is enough space in the inventory for the res, we want to produce.
-		@return bool, true if everything can fit."""
+		@return bool, True if everything can fit."""
 		for res, amount in self._prod_line.produced_res.iteritems():
 			if self.inventory.get_free_space_for(res) < amount:
 				return False
@@ -426,23 +439,21 @@ class Production(WorldObject):
 
 class ChangingProduction(Production):
 	"""Same as Production, but can changes properties of the production line"""
-	def _create_production_line(self, prod_line_id):
-		"""Returns a changeable production line instance"""
-		return ProductionLine(prod_line_id)
 
-	def save(self, db):
-		super(ChangingProduction, self).save(db)
-		self._prod_line.save(db, self.worldid)
+	def save(self, db, owner_id):
+		super(ChangingProduction, self).save(db, owner_id)
+		self._prod_line.save(db, owner_id)
 
-	def _load(self, db, worldid):
-		super(ChangingProduction, self)._load(db, worldid)
-		self._prod_line.load(db, self.worldid)
+	def load(self, db, worldid):
+		super(ChangingProduction, self).load(db, worldid)
+		self._prod_line.load(db, worldid)
 
 
 class SettlerProduction(ChangingProduction):
 	"""For settlers, production behaves different:
 	They produce happiness from the goods they get. They get happy immediately when the get
-	the resource (i.e. they produce at production start)"""
+	the resource (i.e. they produce at production start).
+	It needs to be a changing production since the time can be altered"""
 	def _give_produced_res(self):
 		pass # don't give any resources, when they actually should be given
 
@@ -451,100 +462,18 @@ class SettlerProduction(ChangingProduction):
 		# give the resources when taking away the consumed goods at prod start
 		super(SettlerProduction, self)._give_produced_res()
 
+
 class SingleUseProduction(Production):
 	"""This Production just produces one time, and then finishes.
 	Notification of the finishing is done via production_finished listeners.
 	Use case: Settler getting upgrade material"""
-	def __init__(self, inventory, owner_inventory, prod_line_id, **kwargs):
-		super(SingleUseProduction, self).__init__(inventory=inventory, owner_inventory=owner_inventory, prod_line_id=prod_line_id, **kwargs)
+
+	# TODO: it seems that these kinds of productions are never removed (for settlers and unit productions)
+
+	def __init__(self, inventory, owner_inventory, prod_id, prod_data, **kwargs):
+		super(SingleUseProduction, self).__init__(inventory=inventory, owner_inventory=owner_inventory,
+		                                          prod_id=prod_id, prod_data=prod_data, **kwargs)
 
 	def _finished_producing(self, **kwargs):
 		super(SingleUseProduction, self)._finished_producing(continue_producing=False, **kwargs)
-		self.state = PRODUCTION.STATES.done
-
-class ProgressProduction(Production):
-	"""Same as Production, but starts as soon as any needed res is available (doesn't wait
-	until all of them are ready)
-
-	Implementation:
-	When res are available, they are removed from the production line. The prod line is
-	reloaded on a new production."""
-
-	## INIT/DESTRUCT
-	def __init__(self, **kwargs):
-		super(ProgressProduction, self).__init__(**kwargs)
-		self.__init()
-
-	def __init(self, progress = 0):
-		self.original_prod_line = copy.deepcopy(self._prod_line)
-		self.progress = progress # float indicating current production progress
-
-	def save(self, db):
-		# TODO
-		pass
-
-	def _load(self, db, worldid):
-		super(ProgressProduction, self).load(db, worldid)
-		self.__init()
-		# TODO
-
-	## PROTECTED METHODS
-	def _check_available_res(self):
-		for res in self._prod_line.consumed_res.iterkeys():
-			if self.inventory[res] > 0:
-				return True
-		return False
-
-	def _remove_res_to_expend(self):
-		"""Takes as many res as there are and returns sum of amount of res taken."""
-		taken = 0
-		for res, amount in self._prod_line.consumed_res.iteritems():
-			remnant = self.inventory.alter(res, amount) # try to get all
-			self._prod_line.change_amount(res, remnant) # set how much we still need to get
-			taken += abs(remnant) + amount
-		return taken
-
-	def _produce(self):
-		# check if we're done
-		still_needed_res = sum(self._prod_line.consumed_res.itervalues())
-		if still_needed_res == 0:
-			self._finished_producing()
-			return
-
-		removed_res = self._remove_res_to_expend()
-		# check if there were res
-		if removed_res == 0:
-			# watch inventory for new res
-			self.inventory.add_change_listener(self._check_inventory)
-			if self.__class__.USES_GOLD:
-				self.owner_inventory.add_change_listener(self._check_inventory)
-			self._state = PRODUCTION.STATES.waiting_for_res
-			self._changed()
-			return
-
-		# calculate how much of the whole production process we can produce now
-		# and set the scheduler waiting time accordingly (e.g. half of res => wait half of prod time)
-		all_needed_res = sum(self.original_prod_line.consumed_res.itervalues())
-		part_of_whole_production = float(removed_res) / all_needed_res
-		prod_time = Scheduler().get_ticks( part_of_whole_production * self._prod_line.time )
-		prod_time = min(prod_time, 1) # wait at least 1 tick
-		# do part of production and call this again when done
-		Scheduler().add_new_object(self._produce, self, prod_time)
-
-		# set new progress
-		self.progress += part_of_whole_production
-
-	def _finished_producing(self, **kwargs):
-		super(ProgressProduction, self)._finished_producing(**kwargs)
-		self.progress = 0
-		# reset prodline
-		self._prod_line = copy.copy(self.original_prod_line)
-
-class SingleUseProgressProduction(ProgressProduction, SingleUseProduction):
-	"""A production that needs to have a progress and also is only used one time."""
-
-	def __init__(self, **kwargs):
-		super(SingleUseProgressProduction, self).__init__(**kwargs)
-
-	def _finished_producing(self, **kwargs):
-		super(SingleUseProgressProduction, self)._finished_producing(**kwargs)
+		self._state = PRODUCTION.STATES.done
